{-# LANGUAGE OverloadedStrings #-}

module Main where

import PUBMED.Client
import PUBMED.Parser
import Network.HTTP.Client (newManager)
import Network.HTTP.Client.TLS (tlsManagerSettings)
import Servant.Client (runClientM, mkClientEnv, BaseUrl(..), Scheme(..))
import Text.XML (parseLBS_, def)
import Text.XML.Cursor (fromDocument, Cursor)
import Data.Conduit (ConduitT)
import Data.ByteString.Lazy (ByteString)
import Data.ByteString.Char8 (pack)
import Control.Monad.Catch (MonadThrow)
import Text.Parsec hiding (runParser)
import Text.Parsec.ByteString.Lazy
import qualified Data.ByteString.Lazy as LBS
import qualified Data.Text as T

runParser :: Show res => (Cursor -> res) -> LBS.ByteString -> res
runParser parser = parser . fromDocument . parseLBS_ def

removeSub :: Parser ByteString
removeSub = do
  dt <- many1 anyChar <* (skipMany $ sub <|> asub)
  return . LBS.fromStrict $ pack dt
  where
    sub = string "<sub>"
    asub = string "</sub>"

runSimpleFindPubmedAbstractRequest :: T.Text -> IO [PubMed]
runSimpleFindPubmedAbstractRequest rq = do
  manager' <- newManager tlsManagerSettings
  res <- runClientM
    (search (Just rq))
    (mkClientEnv manager' $ BaseUrl Https "eutils.ncbi.nlm.nih.gov" 443 "entrez/eutils")
  case res of
    (Left err) -> return []
    (Right (BsXml docs)) -> do
      let docIds = runParser parseDocId docs
      res' <- runClientM
        (fetch (Just "pubmed") (Just "abstract") docIds)
        (mkClientEnv manager' $ BaseUrl Https "eutils.ncbi.nlm.nih.gov" 443 "entrez/eutils")
      case res' of
        (Left err) -> return []
        (Right (BsXml abstracts)) -> do
          case (parse removeSub "error" abstracts) of
            (Left _) -> return []
            (Right v) -> pubMedParser v

main :: IO ()
main = do
  pubmeds <- runSimpleFindPubmedAbstractRequest "bisphenol"
  print pubmeds
